@@ -30,24 +30,20 @@
 #include <string.h>
 #include <stdlib.h>
 
+#undef G_DISABLE_DEPRECATED /* for GCompletion */
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
 #include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
-#ifdef ENABLE_TPL
 #include <telepathy-logger/log-manager.h>
-#else
-
-#include <libempathy/empathy-log-manager.h>
-#endif /* ENABLE_TPL */
 #include <libempathy/empathy-contact-list.h>
+#include <libempathy/empathy-gsettings.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-dispatcher.h>
 
 #include "empathy-chat.h"
-#include "empathy-conf.h"
 #include "empathy-spell.h"
 #include "empathy-contact-list-store.h"
 #include "empathy-contact-list-view.h"
@@ -68,7 +64,7 @@
 #define COMPOSING_STOP_TIMEOUT 5
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyChat)
-typedef struct {
+struct _EmpathyChatPriv {
 	EmpathyTpChat     *tp_chat;
 	TpAccount         *account;
 	gchar             *id;
@@ -77,11 +73,10 @@ typedef struct {
 	EmpathyContact    *remote_contact;
 	gboolean           show_contacts;
 
-#ifdef ENABLE_TPL
+	GSettings         *gsettings_chat;
+	GSettings         *gsettings_ui;
+
 	TplLogManager     *log_manager;
-#else
-	EmpathyLogManager *log_manager;
-#endif /* ENABLE_TPL */
 	TpAccountManager  *account_manager;
 	GList             *input_history;
 	GList             *input_history_current;
@@ -92,6 +87,17 @@ typedef struct {
 	TpHandleType       handle_type;
 	gint               contacts_width;
 	gboolean           has_input_vscroll;
+	gint               topic_width;
+
+	/* TRUE if spell checking is enabled, FALSE otherwise.
+	 * This is to keep track of the last state of spell checking
+	 * when it changes. */
+	gboolean	   spell_checking_enabled;
+
+	/* These store the signal handler ids for the enclosed text entry. */
+	gulong		   insert_text_id;
+	gulong		   delete_range_id;
+	gulong		   notify_cursor_position_id;
 
 	GtkWidget         *widget;
 	GtkWidget         *hpaned;
@@ -100,6 +106,7 @@ typedef struct {
 	GtkWidget         *scrolled_window_input;
 	GtkWidget         *scrolled_window_contacts;
 	GtkWidget         *hbox_topic;
+	GtkWidget         *expander_topic;
 	GtkWidget         *label_topic;
 	GtkWidget         *contact_list_view;
 	GtkWidget         *info_bar_vbox;
@@ -132,7 +139,7 @@ typedef struct {
 	 * notified again about the already notified pending messages when the
 	 * messages in tab will be properly shown */
 	gboolean           retrieving_backlogs;
-} EmpathyChatPriv;
+};
 
 typedef struct {
 	gchar *text; /* Original message that was specified
@@ -220,28 +227,6 @@ chat_set_property (GObject      *object,
 	};
 }
 
-static void
-chat_connect_channel_reconnected (EmpathyDispatchOperation *dispatch,
-				  const GError             *error,
-				  gpointer                  user_data)
-{
-	EmpathyChat *chat = EMPATHY_CHAT (user_data);
-	EmpathyTpChat *tpchat;
-
-	if (error != NULL) {
-		empathy_chat_view_append_event (chat->view,
-			_("Failed to reconnect this chat"));
-		return;
-	}
-
-	tpchat = EMPATHY_TP_CHAT (
-		empathy_dispatch_operation_get_channel_wrapper (dispatch));
-
-	if (empathy_dispatch_operation_claim (dispatch)) {
-		empathy_chat_set_tp_chat (chat, tpchat);
-	}
-}
-
 static void
 reconnected_connection_ready_cb (TpConnection *connection,
 			const GError *error,
@@ -257,18 +242,18 @@ reconnected_connection_ready_cb (TpConnection *connection,
 
 	DEBUG ("Account reconnected, request a new Text channel");
 
+	/* FIXME: Ideally we should ask to handle ourself the channel so we can
+	* report the error if any but this is blocked by
+	* https://bugs.freedesktop.org/show_bug.cgi?id=13422 */
 	switch (priv->handle_type) {
 		case TP_HANDLE_TYPE_CONTACT:
 			empathy_dispatcher_chat_with_contact_id (
-				connection, priv->id,
-				chat_connect_channel_reconnected,
-				chat);
+				connection, priv->id, EMPATHY_DISPATCHER_NON_USER_ACTION,
+				NULL, NULL);
 			break;
 		case TP_HANDLE_TYPE_ROOM:
 			empathy_dispatcher_join_muc (connection,
-				priv->id,
-				chat_connect_channel_reconnected,
-				chat);
+				priv->id, EMPATHY_DISPATCHER_NON_USER_ACTION);
 			break;
 		default:
 			g_assert_not_reached ();
@@ -642,25 +627,12 @@ chat_input_history_update (EmpathyChat *chat,
 	g_free (text);
 }
 
-static void
-chat_command_join_cb (EmpathyDispatchOperation *dispatch,
-		      const GError             *error,
-		      gpointer                  user_data)
-{
-	EmpathyChat *chat = user_data;
-
-	if (error != NULL) {
-		DEBUG ("Error: %s", error->message);
-		empathy_chat_view_append_event (chat->view,
-			_("Failed to join chat room"));
-	}
-}
-
 typedef struct {
 	EmpathyChat *chat;
 	gchar *message;
 } ChatCommandMsgData;
 
+#if 0
 static void
 chat_command_msg_cb (EmpathyDispatchOperation *dispatch,
 			      const GError             *error,
@@ -690,6 +662,7 @@ OUT:
 	g_free (data->message);
 	g_slice_free (ChatCommandMsgData, data);
 }
+#endif
 
 static void
 chat_command_clear (EmpathyChat *chat,
@@ -734,6 +707,9 @@ chat_command_join (EmpathyChat *chat,
 
 	GStrv rooms = g_strsplit_set (strv[1], ", ", -1);
 
+	/* FIXME: Ideally we should ask to handle ourself the channel so we can
+	* report the error if any but this is blocked by
+	* https://bugs.freedesktop.org/show_bug.cgi?id=13422 */
 	while (rooms[i] != NULL) {
 		/* ignore empty strings */
 		if (!EMP_STR_EMPTY (rooms[i])) {
@@ -741,14 +717,14 @@ chat_command_join (EmpathyChat *chat,
 
 			connection = empathy_tp_chat_get_connection (priv->tp_chat);
 			empathy_dispatcher_join_muc (connection, rooms[i],
-						     chat_command_join_cb,
-						     chat);
+						     gtk_get_current_event_time ());
 		}
 		i++;
 	}
 	g_strfreev (rooms);
 }
 
+#if 0
 static void
 chat_command_msg_internal (EmpathyChat *chat,
 			   const gchar *contact_id,
@@ -765,6 +741,7 @@ chat_command_msg_internal (EmpathyChat *chat,
 	data->message = g_strdup (message);
 	connection = empathy_tp_chat_get_connection (priv->tp_chat);
 	empathy_dispatcher_chat_with_contact_id (connection, contact_id,
+						 gtk_get_current_event_time (),
 						 chat_command_msg_cb,
 						 data);
 }
@@ -784,6 +761,7 @@ chat_command_msg (EmpathyChat *chat,
 {
 	chat_command_msg_internal (chat, strv[1], strv[2]);
 }
+#endif
 
 static void
 chat_command_nick (EmpathyChat *chat,
@@ -855,11 +833,14 @@ static ChatCommandItem commands[] = {
 	{"j", 2, 2, chat_command_join,
 	 N_("/j <chat room ID>: join a new chat room")},
 
+#if 0
+/* FIXME: https://bugzilla.gnome.org/show_bug.cgi?id=623682 */
 	{"query", 2, 3, chat_command_query,
 	 N_("/query <contact ID> [<message>]: open a private chat")},
 
 	{"msg", 3, 3, chat_command_msg,
 	 N_("/msg <contact ID> <message>: open a private chat")},
+#endif
 
 	{"nick", 2, 2, chat_command_nick,
 	 N_("/nick <nickname>: change your nickname on the current server")},
@@ -1138,7 +1119,7 @@ chat_message_received (EmpathyChat *chat, EmpathyMessage *message)
 	sender = empathy_message_get_sender (message);
 
 	DEBUG ("Appending new message from %s (%d)",
-		empathy_contact_get_name (sender),
+		empathy_contact_get_alias (sender),
 		empathy_contact_get_handle (sender));
 
 	empathy_chat_view_append_message (chat->view, message);
@@ -1197,6 +1178,88 @@ chat_send_error_cb (EmpathyTpChat          *tp_chat,
 	g_free (str);
 }
 
+/* WARNING: EXPLICIT CONTENT, keep away childrens!
+ *
+ * When a GtkLabel is set to wrap, it assume and hardcoded width. To change
+ * that width we have to set a size request on the label... but that's not
+ * possible because we want the window to be able to shrink, so we MUST request
+ * width of 1. Note that the height of a wrapping label depends on its width.
+ *
+ * To work around that, here is what happens:
+ * 1) size-request is first called, an hardcoded small width is requested by
+ *    GtkLabel, which means also a too big height. We do nothing.
+ * 2) size-allocate is called with the full width available, that's the width
+ *    we really want to make wrap the label. We save that width and restart a
+ *    size-request/size-allocate round.
+ * 3) size-request is called a 2nd time, now we can tell the pango layout its
+ *    width (we can't do that in step 2 because GtkLabel::size-request recreate
+ *    the layout each time). When the layout has its width, we can know the
+ *    height of the label and set its requisition. The width request is set to
+ *    1px to make sure the window can shrink, the layout will fill all the
+ *    available width anyway.
+ */
+static void
+chat_topic_label_size_request_cb (GtkLabel *label,
+				   GtkRequisition *requisition,
+				   EmpathyChat *chat)
+{
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+
+	if (gtk_label_get_line_wrap (label) && priv->topic_width > 0) {
+		PangoLayout *layout;
+		PangoRectangle rect;
+		gint ypad;
+
+		layout = gtk_label_get_layout (label);
+		pango_layout_set_width (layout, priv->topic_width * PANGO_SCALE);
+		pango_layout_get_extents (layout, NULL, &rect);
+		gtk_misc_get_padding (GTK_MISC (label), NULL, &ypad);
+
+		requisition->width = 1;
+		requisition->height = PANGO_PIXELS (rect.height) + ypad * 2;
+	}
+}
+
+static void
+chat_topic_label_size_allocate_cb (GtkLabel *label,
+				   GtkAllocation *allocation,
+				   EmpathyChat *chat)
+{
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+
+	if (!gtk_label_get_line_wrap (label)) {
+		priv->topic_width = -1;
+
+		if (pango_layout_is_ellipsized (gtk_label_get_layout (label)))
+			gtk_widget_show (priv->expander_topic);
+		else
+			gtk_widget_hide (priv->expander_topic);
+
+		return;
+	}
+
+	if (priv->topic_width != allocation->width) {
+		priv->topic_width = allocation->width;
+		gtk_widget_queue_resize (GTK_WIDGET (label));
+	}
+}
+
+static void
+chat_topic_expander_activate_cb (GtkExpander *expander,
+				 GParamSpec *param_spec,
+				 EmpathyChat *chat)
+{
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+
+	if (gtk_expander_get_expanded (expander)) {
+		gtk_label_set_ellipsize (GTK_LABEL (priv->label_topic), PANGO_ELLIPSIZE_NONE);
+		gtk_label_set_line_wrap (GTK_LABEL (priv->label_topic), TRUE);
+	} else {
+		gtk_label_set_ellipsize (GTK_LABEL (priv->label_topic), PANGO_ELLIPSIZE_END);
+		gtk_label_set_line_wrap (GTK_LABEL (priv->label_topic), FALSE);
+	}
+}
+
 static void
 chat_property_changed_cb (EmpathyTpChat *tp_chat,
 			  const gchar   *name,
@@ -1213,11 +1276,16 @@ chat_property_changed_cb (EmpathyTpChat *tp_chat,
 		if (EMP_STR_EMPTY (priv->subject)) {
 			gtk_widget_hide (priv->hbox_topic);
 		} else {
+			gchar *markup_topic;
 			gchar *markup_text;
 
-			markup_text = empathy_add_link_markup (priv->subject);
+			markup_topic = empathy_add_link_markup (priv->subject);
+			markup_text = g_strdup_printf ("<span weight=\"bold\">%s</span> %s",
+				_("Topic:"), markup_topic);
+
 			gtk_label_set_markup (GTK_LABEL (priv->label_topic), markup_text);
 			g_free (markup_text);
+			g_free (markup_topic);
 
 			gtk_widget_show (priv->hbox_topic);
 		}
@@ -1240,80 +1308,160 @@ chat_property_changed_cb (EmpathyTpChat *tp_chat,
 	}
 }
 
+static gboolean
+chat_input_text_get_word_from_iter (GtkTextIter   *iter,
+                                    GtkTextIter   *start,
+                                    GtkTextIter   *end)
+{
+	GtkTextIter word_start = *iter;
+	GtkTextIter word_end = *iter;
+	GtkTextIter tmp;
+
+	if (gtk_text_iter_inside_word (&word_end) &&
+			!gtk_text_iter_ends_word (&word_end)) {
+		gtk_text_iter_forward_word_end (&word_end);
+	}
+
+	tmp = word_end;
+
+	if (gtk_text_iter_get_char (&tmp) == '\'') {
+		gtk_text_iter_forward_char (&tmp);
+
+		if (g_unichar_isalpha (gtk_text_iter_get_char (&tmp))) {
+			gtk_text_iter_forward_word_end (&word_end);
+		}
+	}
+
+
+	if (gtk_text_iter_inside_word (&word_start) ||
+			gtk_text_iter_ends_word (&word_start)) {
+		if (!gtk_text_iter_starts_word (&word_start) ||
+				gtk_text_iter_equal (&word_start, &word_end)) {
+			gtk_text_iter_backward_word_start (&word_start);
+		}
+
+		tmp = word_start;
+		gtk_text_iter_backward_char (&tmp);
+
+		if (gtk_text_iter_get_char (&tmp) == '\'') {
+			gtk_text_iter_backward_char (&tmp);
+
+			if (g_unichar_isalpha (gtk_text_iter_get_char (&tmp))) {
+				gtk_text_iter_backward_word_start (&word_start);
+			}
+		}
+	}
+
+	*start = word_start;
+	*end = word_end;
+	return TRUE;
+}
+
+static void
+chat_input_text_buffer_insert_text_cb (GtkTextBuffer *buffer,
+                                       GtkTextIter   *location,
+                                       gchar         *text,
+                                       gint           len,
+                                       EmpathyChat   *chat)
+{
+	GtkTextIter iter, pos;
+
+	/* Remove all misspelled tags in the inserted text.
+	 * This happens when text is inserted within a misspelled word. */
+	gtk_text_buffer_get_iter_at_offset (buffer, &iter,
+					    gtk_text_iter_get_offset (location) - len);
+	gtk_text_buffer_remove_tag_by_name (buffer, "misspelled",
+					    &iter, location);
+
+	gtk_text_buffer_get_iter_at_mark (buffer, &pos, gtk_text_buffer_get_insert (buffer));
+
+	do {
+		GtkTextIter start, end;
+		gchar *str;
+
+		if (!chat_input_text_get_word_from_iter (&iter, &start, &end))
+			continue;
+
+		str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+
+		if (gtk_text_iter_in_range (&pos, &start, &end) ||
+				gtk_text_iter_equal (&pos, &end) ||
+				empathy_spell_check (str)) {
+			gtk_text_buffer_remove_tag_by_name (buffer, "misspelled", &start, &end);
+		} else {
+			gtk_text_buffer_apply_tag_by_name (buffer, "misspelled", &start, &end);
+		}
+
+		g_free (str);
+
+	} while (gtk_text_iter_forward_word_end (&iter) &&
+		 gtk_text_iter_compare (&iter, location) <= 0);
+}
+
+static void
+chat_input_text_buffer_delete_range_cb (GtkTextBuffer *buffer,
+                                        GtkTextIter   *start,
+                                        GtkTextIter   *end,
+                                        EmpathyChat   *chat)
+{
+	GtkTextIter word_start, word_end;
+
+	if (chat_input_text_get_word_from_iter (start, &word_start, &word_end)) {
+		gtk_text_buffer_remove_tag_by_name (buffer, "misspelled",
+						    &word_start, &word_end);
+	}
+}
+
 static void
 chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer,
                                    EmpathyChat    *chat)
 {
-	GtkTextIter     start, end;
-	gchar          *str;
-	gboolean        spell_checker = FALSE;
-
 	if (gtk_text_buffer_get_char_count (buffer) == 0) {
 		chat_composing_stop (chat);
 	} else {
 		chat_composing_start (chat);
 	}
+}
 
-	empathy_conf_get_bool (empathy_conf_get (),
-                           EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED,
-                           &spell_checker);
+static void
+chat_input_text_buffer_notify_cursor_position_cb (GtkTextBuffer *buffer,
+                                                  GParamSpec    *pspec,
+                                                  EmpathyChat    *chat)
+{
+	GtkTextIter pos;
+	GtkTextIter prev_pos;
+	GtkTextIter word_start;
+	GtkTextIter word_end;
+	GtkTextMark *mark;
+	gchar *str;
 
-	gtk_text_buffer_get_start_iter (buffer, &start);
+	mark = gtk_text_buffer_get_mark (buffer, "previous-cursor-position");
 
-	if (!spell_checker) {
-		gtk_text_buffer_get_end_iter (buffer, &end);
-		gtk_text_buffer_remove_tag_by_name (buffer, "misspelled", &start, &end);
-		return;
-	}
+	gtk_text_buffer_get_iter_at_mark (buffer, &pos,
+					  gtk_text_buffer_get_insert (buffer));
+	gtk_text_buffer_get_iter_at_mark (buffer, &prev_pos, mark);
 
-	if (!empathy_spell_supported ()) {
-		return;
-	}
+	if (!chat_input_text_get_word_from_iter (&prev_pos, &word_start, &word_end))
+		goto out;
 
-	/* NOTE: this is really inefficient, we shouldn't have to
-	   reiterate the whole buffer each time and check each work
-	   every time. */
-	while (TRUE) {
-		gboolean correct = FALSE;
+	if (!gtk_text_iter_in_range (&pos, &word_start, &word_end) &&
+			!gtk_text_iter_equal (&pos, &word_end)) {
+		str = gtk_text_buffer_get_text (buffer,
+					&word_start, &word_end, FALSE);
 
-		/* if at start */
-		if (gtk_text_iter_is_start (&start)) {
-			end = start;
-
-			if (!gtk_text_iter_forward_word_end (&end)) {
-				/* no whole word yet */
-				break;
-			}
+		if (!empathy_spell_check (str)) {
+			gtk_text_buffer_apply_tag_by_name (buffer,
+					"misspelled", &word_start, &word_end);
 		} else {
-			if (!gtk_text_iter_forward_word_end (&end)) {
-				/* must be the end of the buffer */
-				break;
-			}
-
-			start = end;
-			gtk_text_iter_backward_word_start (&start);
-		}
-
-		str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-
-		/* spell check string if not a command */
-		if (str[0] != '/') {
-			correct = empathy_spell_check (str);
-		} else {
-			correct = TRUE;
-		}
-
-		if (!correct) {
-			gtk_text_buffer_apply_tag_by_name (buffer, "misspelled", &start, &end);
-		} else {
-			gtk_text_buffer_remove_tag_by_name (buffer, "misspelled", &start, &end);
+			gtk_text_buffer_remove_tag_by_name (buffer,
+					"misspelled", &word_start, &word_end);
 		}
 
 		g_free (str);
-
-		/* set start iter to the end iters position */
-		start = end;
 	}
+
+out:
+	gtk_text_buffer_move_mark (buffer, mark, &pos);
 }
 
 static gboolean
@@ -1374,8 +1522,8 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 		 */
 
 		view = GTK_TEXT_VIEW (chat->input_text_view);
-		if (gtk_im_context_filter_keypress (view->im_context, event)) {
-			GTK_TEXT_VIEW (chat->input_text_view)->need_im_reset = TRUE;
+		if (gtk_text_view_im_context_filter_keypress (view, event)) {
+			gtk_text_view_reset_im_context (view);
 			return TRUE;
 		}
 
@@ -1438,7 +1586,6 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 		if (completed) {
 			guint        len;
 			const gchar *text;
-			gchar       *complete_char = NULL;
 			GString     *message = NULL;
 			GList       *l;
 
@@ -1453,7 +1600,7 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 				 * which might be cased all wrong.
 				 * Fixes #120876
 				 * */
-				text = empathy_contact_get_name (completed_list->data);
+				text = empathy_contact_get_alias (completed_list->data);
 			} else {
 				text = completed;
 
@@ -1463,7 +1610,7 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 				 * */
 				 message = g_string_new ("");
 				 for (l = completed_list; l != NULL; l = l->next) {
-					g_string_append (message, empathy_contact_get_name (l->data));
+					g_string_append (message, empathy_contact_get_alias (l->data));
 					g_string_append (message, " - ");
 				 }
 				 empathy_chat_view_append_event (chat->view, message->str);
@@ -1472,16 +1619,20 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 
 			gtk_text_buffer_insert_at_cursor (buffer, text, strlen (text));
 
-			if (len == 1 && is_start_of_buffer &&
-			    empathy_conf_get_string (empathy_conf_get (),
-						     EMPATHY_PREFS_CHAT_NICK_COMPLETION_CHAR,
-						     &complete_char) &&
-			    complete_char != NULL) {
+			if (len == 1 && is_start_of_buffer) {
+			    gchar *complete_char;
+
+			    complete_char = g_settings_get_string (
+				    priv->gsettings_chat,
+				    EMPATHY_PREFS_CHAT_NICK_COMPLETION_CHAR);
+
+			    if (complete_char != NULL) {
 				gtk_text_buffer_insert_at_cursor (buffer,
 								  complete_char,
 								  strlen (complete_char));
 				gtk_text_buffer_insert_at_cursor (buffer, " ", 1);
 				g_free (complete_char);
+			    }
 			}
 
 			g_free (completed);
@@ -1743,43 +1894,28 @@ chat_input_populate_popup_cb (GtkTextView *view,
 }
 
 
-#ifdef ENABLE_TPL
 static gboolean
-chat_log_filter (TplLogEntry *log,
+chat_log_filter (TplEntry *log,
 		 gpointer user_data)
-#else
-static gboolean
-chat_log_filter (EmpathyMessage *message,
-		 gpointer user_data)
-#endif /* ENABLE_TPL */
 {
 	EmpathyChat *chat = user_data;
-#ifdef ENABLE_TPL
 	EmpathyMessage *message;
-#endif /* ENABLE_TPL */
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	const GList *pending;
 
-#ifdef ENABLE_TPL
-	g_return_val_if_fail (TPL_IS_LOG_ENTRY (log), FALSE);
-#else
-	g_return_val_if_fail (EMPATHY_IS_MESSAGE (message), FALSE);
-#endif /* ENABLE_TPL */
+	g_return_val_if_fail (TPL_IS_ENTRY (log), FALSE);
 	g_return_val_if_fail (EMPATHY_IS_CHAT (chat), FALSE);
 
 	pending = empathy_tp_chat_get_pending_messages (priv->tp_chat);
-#ifdef ENABLE_TPL
 	message = empathy_message_from_tpl_log_entry (log);
-#endif /* ENABLE_TPL */
 
 	for (; pending; pending = g_list_next (pending)) {
 		if (empathy_message_equal (message, pending->data)) {
 			return FALSE;
 		}
 	}
-#ifdef ENABLE_TPL
+
 	g_object_unref (message);
-#endif
 	return TRUE;
 }
 
@@ -1806,7 +1942,6 @@ show_pending_messages (EmpathyChat *chat) {
 }
 
 
-#ifdef ENABLE_TPL
 static void
 got_filtered_messages_cb (GObject *manager,
 		GAsyncResult *result,
@@ -1818,9 +1953,8 @@ got_filtered_messages_cb (GObject *manager,
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	GError *error = NULL;
 
-	messages = tpl_log_manager_get_filtered_messages_async_finish (result, &error);
-
-	if (error != NULL) {
+	if (!tpl_log_manager_get_filtered_messages_finish (TPL_LOG_MANAGER (manager),
+		result, &messages, &error)) {
 		DEBUG ("%s. Aborting.", error->message);
 		empathy_chat_view_append_event (chat->view,
 			_("Failed to retrieve recent logs"));
@@ -1830,7 +1964,7 @@ got_filtered_messages_cb (GObject *manager,
 
 	for (l = messages; l; l = g_list_next (l)) {
 		EmpathyMessage *message;
-		g_assert (TPL_IS_LOG_ENTRY (l->data));
+		g_assert (TPL_IS_ENTRY (l->data));
 
 		message = empathy_message_from_tpl_log_entry (l->data);
 		g_object_unref (l->data);
@@ -1853,17 +1987,12 @@ out:
 	/* Turn back on scrolling */
 	empathy_chat_view_scroll (chat->view, TRUE);
 }
-#endif /* ENABLE_TPL */
-
 
 static void
 chat_add_logs (EmpathyChat *chat)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	gboolean         is_chatroom;
-#ifndef ENABLE_TPL
-	GList           *messages, *l;
-#endif /* ENABLE_TPL */
 
 	if (!priv->id) {
 		return;
@@ -1875,25 +2004,6 @@ chat_add_logs (EmpathyChat *chat)
 	/* Add messages from last conversation */
 	is_chatroom = priv->handle_type == TP_HANDLE_TYPE_ROOM;
 
-#ifndef ENABLE_TPL
-	messages = empathy_log_manager_get_filtered_messages (priv->log_manager,
-							      priv->account,
-							      priv->id,
-							      is_chatroom,
-							      5,
-							      chat_log_filter,
-							      chat);
-
-	for (l = messages; l; l = g_list_next (l)) {
-		empathy_chat_view_append_message (chat->view, l->data);
-		g_object_unref (l->data);
-	}
-
-	g_list_free (messages);
-
-	/* Turn back on scrolling */
-	empathy_chat_view_scroll (chat->view, TRUE);
-#else
 	priv->retrieving_backlogs = TRUE;
 	tpl_log_manager_get_filtered_messages_async (priv->log_manager,
 							      priv->account,
@@ -1904,7 +2014,6 @@ chat_add_logs (EmpathyChat *chat)
 							      chat,
 							      got_filtered_messages_cb,
 							      (gpointer) chat);
-#endif /* ENABLE_TPL */
 }
 
 static gint
@@ -1948,7 +2057,7 @@ build_part_message (guint           reason,
 	const gchar *actor_name = NULL;
 
 	if (actor != NULL) {
-		actor_name = empathy_contact_get_name (actor);
+		actor_name = empathy_contact_get_alias (actor);
 	}
 
 	/* Having an actor only really makes sense for a few actions... */
@@ -2004,7 +2113,7 @@ chat_members_changed_cb (EmpathyTpChat  *tp_chat,
 			 EmpathyChat    *chat)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
-	const gchar *name = empathy_contact_get_name (contact);
+	const gchar *name = empathy_contact_get_alias (contact);
 	gchar *str;
 
 	g_return_if_fail (TP_CHANNEL_GROUP_CHANGE_REASON_RENAMED != reason);
@@ -2039,8 +2148,8 @@ chat_member_renamed_cb (EmpathyTpChat  *tp_chat,
 		gchar *str;
 
 		str = g_strdup_printf (_("%s is now known as %s"),
-				       empathy_contact_get_name (old_contact),
-				       empathy_contact_get_name (new_contact));
+				       empathy_contact_get_alias (old_contact),
+				       empathy_contact_get_alias (new_contact));
 		empathy_chat_view_append_event (chat->view, str);
 		g_free (str);
 	}
@@ -2090,7 +2199,11 @@ chat_update_contacts_visibility (EmpathyChat *chat,
 						priv->contacts_width);
 		}
 
-		store = empathy_contact_list_store_new (EMPATHY_CONTACT_LIST (priv->tp_chat));
+		store = empathy_contact_list_store_new (
+				EMPATHY_CONTACT_LIST (priv->tp_chat));
+		empathy_contact_list_store_set_show_groups (
+				EMPATHY_CONTACT_LIST_STORE (store), FALSE);
+
 		priv->contact_list_view = GTK_WIDGET (empathy_contact_list_view_new (store,
 			EMPATHY_CONTACT_LIST_FEATURE_CONTACT_TOOLTIP,
 			EMPATHY_CONTACT_FEATURE_CHAT |
@@ -2180,13 +2293,118 @@ chat_destroy_cb (EmpathyTpChat *tp_chat,
 }
 
 static gboolean
-chat_hpaned_pos_changed_cb (GtkWidget* hpaned, gpointer user_data)
+update_misspelled_words (gpointer data)
 {
+	EmpathyChat *chat = EMPATHY_CHAT (data);
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+	gint length;
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
+
+	gtk_text_buffer_get_end_iter (buffer, &iter);
+	length = gtk_text_iter_get_offset (&iter);
+	chat_input_text_buffer_insert_text_cb (buffer, &iter,
+					       NULL, length, chat);
+	return FALSE;
+}
+
+static void
+conf_spell_checking_cb (GSettings *gsettings_chat,
+			const gchar *key,
+			gpointer user_data)
+{
+	EmpathyChat *chat = EMPATHY_CHAT (user_data);
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+	gboolean spell_checker;
+	GtkTextBuffer *buffer;
+
+	if (strcmp (key, EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED) != 0)
+		return;
+
+	spell_checker = g_settings_get_boolean (gsettings_chat,
+			EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED);
+
+	if (!empathy_spell_supported ()) {
+		spell_checker = FALSE;
+	}
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
+
+	if (spell_checker == priv->spell_checking_enabled) {
+		if (spell_checker) {
+			/* Possibly changed dictionaries,
+			 * update misspelled words. Need to do so in idle
+			 * so the spell checker is updated. */
+			g_idle_add (update_misspelled_words, chat);
+		}
+
+		return;
+	}
+
+	if (spell_checker) {
+		GtkTextIter iter;
+
+		priv->notify_cursor_position_id = tp_g_signal_connect_object  (
+				buffer, "notify::cursor-position",
+				G_CALLBACK (chat_input_text_buffer_notify_cursor_position_cb),
+				chat, 0);
+		priv->insert_text_id = tp_g_signal_connect_object  (
+				buffer, "insert-text",
+				G_CALLBACK (chat_input_text_buffer_insert_text_cb),
+				chat, G_CONNECT_AFTER);
+		priv->delete_range_id = tp_g_signal_connect_object  (
+				buffer, "delete-range",
+				G_CALLBACK (chat_input_text_buffer_delete_range_cb),
+				chat, G_CONNECT_AFTER);
+
+		gtk_text_buffer_create_tag (buffer, "misspelled",
+					    "underline", PANGO_UNDERLINE_ERROR,
+					    NULL);
+
+		gtk_text_buffer_get_iter_at_mark (buffer, &iter,
+	                                          gtk_text_buffer_get_insert (buffer));
+		gtk_text_buffer_create_mark (buffer, "previous-cursor-position",
+					     &iter, TRUE);
+
+		/* Mark misspelled words in the existing buffer.
+		 * Need to do so in idle so the spell checker is updated. */
+		g_idle_add (update_misspelled_words, chat);
+	} else {
+		GtkTextTagTable *table;
+		GtkTextTag *tag;
+
+		g_signal_handler_disconnect (buffer, priv->notify_cursor_position_id);
+		priv->notify_cursor_position_id = 0;
+		g_signal_handler_disconnect (buffer, priv->insert_text_id);
+		priv->insert_text_id = 0;
+		g_signal_handler_disconnect (buffer, priv->delete_range_id);
+		priv->delete_range_id = 0;
+
+		table = gtk_text_buffer_get_tag_table (buffer);
+		tag = gtk_text_tag_table_lookup (table, "misspelled");
+		gtk_text_tag_table_remove (table, tag);
+
+		gtk_text_buffer_delete_mark_by_name (buffer,
+						     "previous-cursor-position");
+	}
+
+	priv->spell_checking_enabled = spell_checker;
+}
+
+static gboolean
+chat_hpaned_pos_changed_cb (GtkWidget* hpaned,
+		GParamSpec *spec,
+		gpointer user_data)
+{
+	EmpathyChat *chat = EMPATHY_CHAT (user_data);
 	gint hpaned_pos;
+
 	hpaned_pos = gtk_paned_get_position (GTK_PANED(hpaned));
-	empathy_conf_set_int (empathy_conf_get (),
-			      EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS,
-			      hpaned_pos);
+	g_settings_set_int (chat->priv->gsettings_ui,
+			    EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS,
+			    hpaned_pos);
+
 	return TRUE;
 }
 
@@ -2209,10 +2427,18 @@ chat_create_ui (EmpathyChat *chat)
 					"scrolled_window_chat", &priv->scrolled_window_chat,
 					"scrolled_window_input", &priv->scrolled_window_input,
 					"hbox_topic", &priv->hbox_topic,
+					"expander_topic", &priv->expander_topic,
 					"label_topic", &priv->label_topic,
 					"scrolled_window_contacts", &priv->scrolled_window_contacts,
 					"info_bar_vbox", &priv->info_bar_vbox,
 					NULL);
+
+	empathy_builder_connect (gui, chat,
+		"expander_topic", "notify::expanded", chat_topic_expander_activate_cb,
+		"label_topic", "size-allocate", chat_topic_label_size_allocate_cb,
+		"label_topic", "size-request", chat_topic_label_size_request_cb,
+		NULL);
+
 	g_free (filename);
 
 	/* Add message view. */
@@ -2252,12 +2478,14 @@ chat_create_ui (EmpathyChat *chat)
 			  G_CALLBACK (chat_input_populate_popup_cb),
 			  chat);
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
-	empathy_signal_connect_weak  (buffer, "changed",
+	tp_g_signal_connect_object  (buffer, "changed",
 			  G_CALLBACK (chat_input_text_buffer_changed_cb),
-			  G_OBJECT (chat));
-	gtk_text_buffer_create_tag (buffer, "misspelled",
-				    "underline", PANGO_UNDERLINE_ERROR,
-				    NULL);
+			  chat, 0);
+	tp_g_signal_connect_object (priv->gsettings_chat,
+			"changed::" EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED,
+			G_CALLBACK (conf_spell_checking_cb), chat, 0);
+	conf_spell_checking_cb (priv->gsettings_chat,
+				EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED, chat);
 	gtk_container_add (GTK_CONTAINER (priv->scrolled_window_input),
 			   chat->input_text_view);
 	gtk_widget_show (chat->input_text_view);
@@ -2274,13 +2502,12 @@ chat_create_ui (EmpathyChat *chat)
 
 	g_signal_connect (priv->hpaned, "notify::position",
 			  G_CALLBACK (chat_hpaned_pos_changed_cb),
-			  NULL);
+			  chat);
 
         /* Load the paned position */
-	if (empathy_conf_get_int (empathy_conf_get (),
-				 EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS,
-				 &paned_pos)
-		&& paned_pos)
+	paned_pos = g_settings_get_int (priv->gsettings_ui,
+			EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS);
+	if (paned_pos != 0)
 		gtk_paned_set_position (GTK_PANED(priv->hpaned), paned_pos);
 
 	/* Set widget focus order */
@@ -2361,6 +2588,9 @@ chat_finalize (GObject *object)
 
 	DEBUG ("Finalized: %p", object);
 
+	g_object_unref (priv->gsettings_chat);
+	g_object_unref (priv->gsettings_ui);
+
 	g_list_foreach (priv->input_history, (GFunc) chat_input_history_entry_free, NULL);
 	g_list_free (priv->input_history);
 
@@ -2415,14 +2645,15 @@ chat_constructed (GObject *object)
 	EmpathyChat *chat = EMPATHY_CHAT (object);
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 
-	if (priv->handle_type != TP_HANDLE_TYPE_ROOM)
+	if (priv->handle_type != TP_HANDLE_TYPE_ROOM) {
+		/* First display logs from the logger and then display pending messages */
 		chat_add_logs (chat);
-#ifndef ENABLE_TPL
-	/* When async API are involved, pending message are shown at the end of the
-	 * callbacks' chain fired by chat_add_logs */
-	priv->can_show_pending = TRUE;
-	show_pending_messages (chat);
-#endif /* ENABLE_TPL */
+	}
+	 else {
+		/* Just display pending messages for rooms */
+		priv->can_show_pending = TRUE;
+		show_pending_messages (chat);
+	}
 }
 
 static void
@@ -2550,9 +2781,9 @@ account_manager_prepared_cb (GObject *source_object,
 
 	for (l = accounts; l != NULL; l = l->next) {
 		TpAccount *account = l->data;
-		empathy_signal_connect_weak (account, "status-changed",
+		tp_g_signal_connect_object (account, "status-changed",
 					     G_CALLBACK (chat_new_connection_cb),
-					     G_OBJECT (chat));
+					     chat, 0);
 	}
 
 	g_list_free (accounts);
@@ -2565,11 +2796,10 @@ empathy_chat_init (EmpathyChat *chat)
 		EMPATHY_TYPE_CHAT, EmpathyChatPriv);
 
 	chat->priv = priv;
-#ifndef ENABLE_TPL
-	priv->log_manager = empathy_log_manager_dup_singleton ();
-#else
 	priv->log_manager = tpl_log_manager_dup_singleton ();
-#endif /* ENABLE_TPL */
+	priv->gsettings_chat = g_settings_new (EMPATHY_PREFS_CHAT_SCHEMA);
+	priv->gsettings_ui = g_settings_new (EMPATHY_PREFS_UI_SCHEMA);
+
 	priv->contacts_width = -1;
 	priv->input_history = NULL;
 	priv->input_history_current = NULL;
@@ -2578,9 +2808,8 @@ empathy_chat_init (EmpathyChat *chat)
 	tp_account_manager_prepare_async (priv->account_manager, NULL,
 					  account_manager_prepared_cb, chat);
 
-	empathy_conf_get_bool (empathy_conf_get (),
-			       EMPATHY_PREFS_CHAT_SHOW_CONTACTS_IN_ROOMS,
-			       &priv->show_contacts);
+	priv->show_contacts = g_settings_get_boolean (priv->gsettings_chat,
+			EMPATHY_PREFS_CHAT_SHOW_CONTACTS_IN_ROOMS);
 
 	/* Block events for some time to avoid having "has come online" or
 	 * "joined" messages. */
@@ -2588,7 +2817,7 @@ empathy_chat_init (EmpathyChat *chat)
 		g_timeout_add_seconds (1, chat_block_events_timeout_cb, chat);
 
 	/* Add nick name completion */
-	priv->completion = g_completion_new ((GCompletionFunc) empathy_contact_get_name);
+	priv->completion = g_completion_new ((GCompletionFunc) empathy_contact_get_alias);
 	g_completion_set_compare (priv->completion, chat_contacts_completion_func);
 
 	chat_create_ui (chat);
@@ -2889,7 +3118,7 @@ empathy_chat_get_name (EmpathyChat *chat)
 
 	ret = priv->name;
 	if (!ret && priv->remote_contact) {
-		ret = empathy_contact_get_name (priv->remote_contact);
+		ret = empathy_contact_get_alias (priv->remote_contact);
 	}
 
 	if (!ret)
@@ -3009,7 +3238,7 @@ empathy_chat_paste (EmpathyChat *chat)
 	priv = GET_PRIV (chat);
 
 	if (priv->tp_chat == NULL ||
-	    !GTK_WIDGET_IS_SENSITIVE (chat->input_text_view))
+	    !gtk_widget_is_sensitive (chat->input_text_view))
 		return;
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
