@@ -203,6 +203,23 @@ tp_chat_get_members (EmpathyContactList *list)
 	return members;
 }
 
+static void
+check_ready (EmpathyTpChat *chat)
+{
+	EmpathyTpChatPriv *priv = GET_PRIV (chat);
+
+	if (priv->ready)
+		return;
+
+	if (g_queue_get_length (priv->messages_queue) > 0)
+		return;
+
+	DEBUG ("Ready");
+
+	priv->ready = TRUE;
+	g_object_notify (G_OBJECT (chat), "ready");
+}
+
 static void
 tp_chat_emit_queued_messages (EmpathyTpChat *chat)
 {
@@ -220,6 +237,8 @@ tp_chat_emit_queued_messages (EmpathyTpChat *chat)
 		g_queue_push_tail (priv->pending_messages_queue, message);
 		g_signal_emit (chat, signals[MESSAGE_RECEIVED], 0, message);
 	}
+
+	check_ready (chat);
 }
 
 static void
@@ -265,6 +284,9 @@ tp_chat_build_message (EmpathyTpChat *chat,
 	empathy_message_set_incoming (message, incoming);
 	empathy_message_set_flags (message, flags);
 
+	if (flags & TP_CHANNEL_TEXT_MESSAGE_FLAG_SCROLLBACK)
+		empathy_message_set_is_backlog (message, TRUE);
+
 	g_queue_push_tail (priv->messages_queue, message);
 
 	if (from_handle == 0) {
@@ -407,7 +429,7 @@ tp_chat_state_changed_got_contact_cb (TpConnection            *connection,
 
 	state = GPOINTER_TO_UINT (user_data);
 	DEBUG ("Chat state changed for %s (%d): %d",
-		empathy_contact_get_name (contact),
+		empathy_contact_get_alias (contact),
 		empathy_contact_get_handle (contact), state);
 
 	g_signal_emit (chat, signals[CHAT_STATE_CHANGED], 0, contact, state);
@@ -495,6 +517,8 @@ tp_chat_list_pending_messages_cb (TpChannel       *channel,
 		acknowledge_messages (chat, empty_non_text_content_ids);
 		g_array_free (empty_non_text_content_ids, TRUE);
 	}
+
+	check_ready (chat);
 }
 
 static void
@@ -797,7 +821,7 @@ tp_chat_finalize (GObject *object)
 }
 
 static void
-tp_chat_check_if_ready (EmpathyTpChat *chat)
+check_almost_ready (EmpathyTpChat *chat)
 {
 	EmpathyTpChatPriv *priv = GET_PRIV (chat);
 
@@ -817,13 +841,14 @@ tp_chat_check_if_ready (EmpathyTpChat *chat)
 	    priv->remote_contact == NULL)
 		return;
 
-	DEBUG ("Ready!");
-
 	tp_cli_channel_type_text_connect_to_received (priv->channel,
 						      tp_chat_received_cb,
 						      NULL, NULL,
 						      G_OBJECT (chat), NULL);
 	priv->listing_pending_messages = TRUE;
+
+	/* TpChat will be ready once ListPendingMessages returned and all the messages
+	 * have been added to the pending messages queue. */
 	tp_cli_channel_type_text_call_list_pending_messages (priv->channel, -1,
 							     FALSE,
 							     tp_chat_list_pending_messages_cb,
@@ -842,8 +867,6 @@ tp_chat_check_if_ready (EmpathyTpChat *chat)
 									   tp_chat_state_changed_cb,
 									   NULL, NULL,
 									   G_OBJECT (chat), NULL);
-	priv->ready = TRUE;
-	g_object_notify (G_OBJECT (chat), "ready");
 }
 
 static void
@@ -942,7 +965,7 @@ tp_chat_got_added_contacts_cb (TpConnection            *connection,
 	}
 
 	tp_chat_update_remote_contact (EMPATHY_TP_CHAT (chat));
-	tp_chat_check_if_ready (EMPATHY_TP_CHAT (chat));
+	check_almost_ready (EMPATHY_TP_CHAT (chat));
 }
 
 static EmpathyContact *
@@ -1052,7 +1075,7 @@ tp_chat_got_renamed_contacts_cb (TpConnection            *connection,
 	}
 
 	tp_chat_update_remote_contact (EMPATHY_TP_CHAT (chat));
-	tp_chat_check_if_ready (EMPATHY_TP_CHAT (chat));
+	check_almost_ready (EMPATHY_TP_CHAT (chat));
 }
 
 
@@ -1148,7 +1171,7 @@ tp_chat_got_remote_contact_cb (TpConnection            *connection,
 	priv->remote_contact = g_object_ref (contact);
 	g_object_notify (chat, "remote-contact");
 
-	tp_chat_check_if_ready (EMPATHY_TP_CHAT (chat));
+	check_almost_ready (EMPATHY_TP_CHAT (chat));
 }
 
 static void
@@ -1168,7 +1191,7 @@ tp_chat_got_self_contact_cb (TpConnection            *connection,
 
 	priv->user = g_object_ref (contact);
 	empathy_contact_set_is_user (priv->user, TRUE);
-	tp_chat_check_if_ready (EMPATHY_TP_CHAT (chat));
+	check_almost_ready (EMPATHY_TP_CHAT (chat));
 }
 
 static void
@@ -1206,7 +1229,7 @@ got_password_flags_cb (TpChannel *proxy,
 	priv->got_password_flags = TRUE;
 	priv->password_flags = password_flags;
 
-	tp_chat_check_if_ready (EMPATHY_TP_CHAT (self));
+	check_almost_ready (EMPATHY_TP_CHAT (self));
 }
 
 static GObject *
@@ -1223,9 +1246,9 @@ tp_chat_constructor (GType                  type,
 	priv = GET_PRIV (chat);
 
 	priv->connection = g_object_ref (tp_channel_borrow_connection (priv->channel));
-	g_signal_connect (priv->channel, "invalidated",
+	tp_g_signal_connect_object (priv->channel, "invalidated",
 			  G_CALLBACK (tp_chat_invalidated_cb),
-			  chat);
+			  chat, 0);
 
 	if (tp_proxy_has_interface_by_id (priv->channel,
 					  TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP)) {
@@ -1247,8 +1270,8 @@ tp_chat_constructor (GType                  type,
 
 		priv->can_upgrade_to_muc = FALSE;
 
-		g_signal_connect (priv->channel, "group-members-changed",
-			G_CALLBACK (tp_chat_group_members_changed_cb), chat);
+		tp_g_signal_connect_object (priv->channel, "group-members-changed",
+			G_CALLBACK (tp_chat_group_members_changed_cb), chat, 0);
 	} else {
 		EmpathyDispatcher *dispatcher = empathy_dispatcher_dup_singleton ();
 		GList *list, *ptr;
@@ -1868,7 +1891,8 @@ empathy_tp_chat_join (EmpathyTpChat *self)
 }
 
 gboolean
-empathy_tp_chat_is_invited (EmpathyTpChat *self)
+empathy_tp_chat_is_invited (EmpathyTpChat *self,
+			    TpHandle *inviter)
 {
 	EmpathyTpChatPriv *priv = GET_PRIV (self);
 	TpHandle self_handle;
@@ -1881,5 +1905,5 @@ empathy_tp_chat_is_invited (EmpathyTpChat *self)
 		return FALSE;
 
 	return tp_channel_group_get_local_pending_info (priv->channel, self_handle,
-		NULL, NULL, NULL);
+		inviter, NULL, NULL);
 }
